<?php

namespace core\admin;
use core\{utility\Utility, theme\Theme, user\User, route\Route, translation\Translation as T};

class Admin {
	function __construct() {
		Utility::set_libraries(__DIR__);
	}

	function admin() {
		$entities_types = $this->get_page();
		$result = (new Theme)->get_template(__DIR__, 'admin', compact('entities_types'));

		return $result;
	}

	function get_page() {
		$routes_data = Route::get_routes('', 'admin/*', '[0-9a-z\.\/\-]*');

		$entities_types = [];
		foreach ($routes_data as $key => $value) {
			$entities_types[$value['entity_type']][$value['entity']][] = $value;
		}

		$map = [
			'core' => T::get('Ядро'),
			'base' => T::get('Базовое'),
			'custom' => T::get('Пользовательское'),
		];

		$menu = [];
		if ($entities_types) {
			$entities_types = [
				'core' => $entities_types['core'] ?? [],
				'base' => $entities_types['base'] ?? [],
				'custom' => $entities_types['custom'] ?? [],
			];

			foreach ($entities_types as $type_name => $entities) {
				$count = 0;
				foreach ($entities as $entity_name => $routes) {
					$entity_upper = ucfirst($entity_name);
					$title = $route_temp = '';

					$on = false;
					$items = [];
					foreach ($routes as $route_index => $route) {
						if ((!isset($route['menu']) || $route['menu']) && (!isset($route['type']) || $route['type'] == 'replace') && (!isset($route['access']) || (new User)->get_access($route['access']))) {
							$first = count(explode('/', $route['route'])) <= 2;

							if (!$title && $first) {
								$title = $route['title'] ?? $entity_upper;
								$route_temp = $route['route'];
							}

							$items[$route['route']] = $route['title'] ?? $entity_upper;

							$on = true;
						}
					}

					if ($on) {
						$menu[$map[$type_name]][$entity_name] = [
							'title' => $title ?: $entity_upper,
							'title_link' => $route_temp ?: 'admin/'.str_replace('_', ' ', $entity_name),
							'items' => $items,
							'tr' => $count && $count % 3 === 0 ? '</tr><tr>' : '',
						];

						$count++;
					}
				}
			}
		}

		return $menu;
	}

	function get_panel() {
		$menu_tree = $this->get_menu();

		$result = '';
		if ($menu_tree) {
			$result = (new Theme)->get_template(__DIR__, 'panel', compact('menu_tree'));
		}

		return $result;
	}

	function get_menu() {
		$menu = [];
		$cache = Utility::check_cache();

		$menu_file = EQSASH_ROOT.'/'.Route::$path_cache.'/menu.php';
		if ($cache && file_exists($menu_file)) {
			require $menu_file;
		} else {
			$menu = $this->make_menu();

			if ($cache) {
				$menu_content = '<?php'.PHP_EOL.PHP_EOL.'$menu = '.var_export($menu, true).';'.PHP_EOL;
				file_put_contents($menu_file, $menu_content);
			}
		}

		return $menu;
	}

	function make_menu() {
		$routes = Route::get_routes('', 'admin*', '[0-9a-z\.\/\-]*');

		$menu_tree = $menu_tree_custom = [];
		foreach ($routes as $key => $value) {
			if ((!isset($value['menu']) || $value['menu']) && (!isset($value['type']) || $value['type'] == 'replace') && (!isset($value['access']) || (new User)->get_access($value['access']))) {
				$variable = $value['entity_type'] == 'custom' ? 'menu_tree_custom' : 'menu_tree';
				$menu = ${$variable};

				$uri_temp_parts = Utility::get_string_parts($value['parent'], '/');
				$uri_temp_parts_count = count($uri_temp_parts) - 1 - Route::get_interim_count();

				if ($uri_temp_parts_count == 1) {
					$value['children'] = $menu[$value['route']]['children'] ?? [];
					$menu[$value['route']] = $value;
				} else if ($uri_temp_parts_count == 2) {
					$value['children'] = $menu[$value['parent']]['children'][$value['route']]['children'] ?? [];
					$menu[$value['parent']]['children'][$value['route']] = $value;
				} else {
					$i = 0;
					for ($i; $i < $uri_temp_parts_count; $i++) {
						array_pop($uri_temp_parts);
						$parent = 'parent_'.$i;
						${$parent} = implode('/', $uri_temp_parts);
					}

					if ($uri_temp_parts_count == 3) {
						$value['children'] = $menu[$parent_0]['children'][$value['parent']]['children'][$value['route']]['children'] ?? [];
						$menu[$parent_0]['children'][$value['parent']]['children'][$value['route']] = $value;
					} else if ($uri_temp_parts_count == 4) {
						$value['children'] = $menu[$parent_1]['children'][$parent_0]['children'][$value['parent']]['children'][$value['route']]['children'] ?? [];
						$menu[$parent_1]['children'][$parent_0]['children'][$value['parent']]['children'][$value['route']] = $value;
					}
				}

				${$variable} = $menu;
			}
		}

		$menu_tree += $menu_tree_custom;

		return $menu_tree;
	}
}
