<?php

namespace core\form;
use core\{utility\Utility, utility\symbol\Symbol, translation\Translation as T};

class Form {
	function make($fields, $buttons = ['submit'], $fieldset = true, $attributes = []) {
		$fields_html = is_array($fields) ? $this->make_field($fields) : $fields;

		$buttons_data = [
			'submit' => [
				'type' => 'submit',
				'value' => 'Подтвердить',
			],
			'cancel' => [
				'type' => 'submit',
				'value' => 'Отменить',
			],
			'reset' => [
				'type' => 'reset',
				'value' => 'Сбросить',
			],
		];

		$buttons_fields = [];
		foreach ($buttons as $key => $value) {
			$input_data = [];
			if (is_array($value)) {
				foreach ($value as $key_two => $value_two) {
					if ($key_two == 'value') {
						$value_two = T::get($value_two);
					}

					$input_data[$key_two] = $value_two;
				}
			} else {
				$input_data = [
					'type' => $buttons_data[$value]['type'],
					'name' => $value,
					'value' => T::get($buttons_data[$value]['value']),
				];
			}

			$button_name = $input_data['name'] ?? 'submit';

			$buttons_fields[$button_name] = [
				'input' => $input_data
			];
		}

		$fields_html .= $this->make_field($buttons_fields);

		if ($fieldset) {
			$form = $this->make_field([
				'fieldset' => [
					'fieldset' => $attributes,
					'value' => [
						'form' => [
							'form' => [],
							'value' => $fields_html,
						],
					],
				],
			]);
		} else {
			$form = $this->make_field([
				'form' => [
					'form' => $attributes,
					'value' => $fields_html,
				],
			]);
		}

		return $form;
	}

	function make_field($data) {
		$multi = false;
		foreach ($data as $key => $value) {
			if (!is_array($value)) {
				continue;
			} else {
				$multi = is_array($value) ? reset($value) : false;
				$multi = is_array($multi);
				break;
			}
		}
		$data = $multi ? $data : ($data ? [$data] : []);

		$result = '';
		foreach ($data as $key => $value) {
			if (isset($value['value']) && is_array($value['value'])) {
				$inside_html = '';
				foreach ($value['value'] as $key_two => $value_two) {
					$inside_html .= is_array($value_two) ? $this->build_field($value_two) : $value_two;
				}

				$value['value'] = $inside_html;
			}

			$result .= is_array($value) ? $this->build_field($value) : $value;
		}

		return $result;
	}

	function build_field($data) {
		$field_type = key($data);

		$title = $description = '';
		if (!empty($data['title'])) {
			$id = $data[$field_type]['id'] ?? 'field_'.$field_type.'_'.Symbol::get_random();
			$data[$field_type]['id'] = $id;

			$required = isset($data[$field_type]['required']) || !empty($data['required']) ? '<span class="required" title="'.T::get('Это поле обязательно для заполнения').'">*</span>' : '';

			$title = '<label class="display-block" for="'.$id.'">'.T::get($data['title']).$required.'</label>';

			unset($data['title']);
		}

		if (!empty($data['description'])) {
			$description = '<div class="field-description">'.T::get($data['description']).'</div>';
			unset($data['description']);
		}

		$attributes = ' ';
		foreach ($data[$field_type] as $key => $value) {
			if ($key == 'placeholder') {
				$value = T::get($value);
			}

			$attributes .= $key.'="'.$value.'" ';
		}
		$attributes = rtrim($attributes);

		$result = $this->{'render_'.$field_type}($data, $attributes, $field_type);
		$result = $title.$result.$description;

		$view = empty($data['view']) ? 'inline' : $data['view'];
		$result = $this->make_view($result, $view);

		return $result;
	}

	function make_view($field, $view) {
		if ($view == 'block') {
			$field = '<div class="field">'.$field.'</div>';
		} else if ($view == 'break') {
			$field = '<div>'.$field.'</div>';
		} else if ($view == 'inline') {
			$field .= ' ';
		} else if ($view == 'none') {
			$field = '<div class="hidden">'.$field.'</div>';
		}

		return $field;
	}

	function set_fields_values($fields, $data) {
		foreach ($fields as $key => $value) {
			$field_value = $value['value'] ?? '';

			if (is_array($field_value)) {
				foreach ($field_value as $key_two => $value_two) {
					if (isset($data[$key_two])) {
						$field_value[$key_two]['value'] = $data[$key_two];
					}
				}

				$value['value'] = $field_value;
			} else {
				if (isset($data[$key]) && is_array($value)) {
					$value['value'] = $data[$key];
				}
			}

			$fields[$key] = $value;
		}

		return $fields;
	}

	//

	function render_fieldset($data, $attributes, $field_type) {
		$legend = isset($data['legend']) ? '<legend>'.T::get($data['legend']).'</legend>' : '';
		$result = '<fieldset'.$attributes.'>'.$legend.$data['value'].'</fieldset>';

		return $result;
	}

	function render_form($data, $attributes, $field_type) {
		$form_value = $data['value'] ?? '';

		$attributes_plus = [];
		if (!isset($data[$field_type]['enctype'])) {
			$attributes_plus['enctype'] = 'multipart/form-data';
		} else if (!$data[$field_type]['enctype']) {
			$attributes_plus['enctype'] = 'application/x-www-form-urlencoded';
		}

		if (!isset($data[$field_type]['enctype'])) {
			$attributes_plus['method'] = 'post';
		}

		if ($attributes_plus) {
			$attributes .= ' ';
			foreach ($attributes_plus as $key => $value) {
				$attributes .= $key.'="'.$value.'" ';
			}
			$attributes = rtrim($attributes);
		}

		$result = '<form'.$attributes.'>'.$form_value.'</form>';

		return $result;
	}

	function render_input($data, $attributes, $field_type) {
		$alter = !isset($data['alter']) || $data['alter'];

		$label = $data['label'] ?? '';
		$label = isset($data['label']) ? ($alter ? T::get($data['label']) : $data['label']) : '';

		$options = $data['options'] ?? [];
		$name = $data[$field_type]['name'] ?? '';
		$name_prefix = isset($data['name_prefix']) ? $data['name_prefix'].'_' : '';

		$input_value = $data['value'] ?? null;
		$input_values = is_array($input_value) ? $input_value : ($input_value !== null ? [$input_value] : []);

		if ($options) {
			$view = empty($data['view']) ? 'inline' : $data['view'];

			$tag = PHP_EOL;
			foreach ($options as $key => $value) {
				$attribute_name = $name ? ' ' : 'name="'.$name_prefix.$key.'" ';
				$checked = in_array($key, $input_values) ? ' checked' : '';

				$options_html = '<input'.$attributes.$attribute_name.'value="'.$key.'"'.$checked.'>'.PHP_EOL;
				$options_html = '<label>'.$options_html.' '.($alter ? T::get($value) : $value).'</label>';
				$options_html = $this->make_view($options_html, $view);

				$tag .= $options_html;
			}
		} else {
			$checked = isset($data[$field_type]['checked']) ? '' : (!empty($data['checked']) ? ' checked' : '');
			$disabled = isset($data[$field_type]['disabled']) ? '' : (!empty($data['disabled']) ? ' disabled' : '');
			$required = isset($data[$field_type]['required']) ? '' : (!empty($data['required']) ? ' required' : '');

			if ($data[$field_type]['type'] == 'checkbox' || $data[$field_type]['type'] == 'radio') {
				$value = isset($data[$field_type]['checked']) || isset($data['checked']) || empty($data['value']) ? '' : ' checked';
			} else {
				$value = isset($data[$field_type]['value']) || !isset($data['value']) ? '' : ' value="'.$data['value'].'"';
			}

			$tag = '<input'.$value.$attributes.$checked.$disabled.$required.'>';
		}

		$result = $label ? '<label>'.$tag.' '.$label.'</label>' : $tag;

		return $result;
	}

	function render_select($data, $attributes, $field_type) {
		$select_no = $data['select_no'] ?? true;
		$alter = !isset($data['alter']) || $data['alter'];

		$select_value = $data['value'] ?? '';
		$options = $data['options'] ?? [];

		$options_html = PHP_EOL;
		$options_html .= $select_no ? '<option value="">'.T::get('Не выбрано').'</option>'.PHP_EOL : '';
		
		foreach ($options as $key => $value) {
			$value = $alter ? T::get($value) : $value;

			$selected = $select_value == $key ? ' selected' : '';
			$options_html .= '<option value="'.$key.'"'.$selected.'>'.$value.'</option>'.PHP_EOL;
		}

		$disabled = isset($data[$field_type]['disabled']) ? '' : (!empty($data['disabled']) ? ' disabled' : '');
		$required = isset($data[$field_type]['required']) ? '' : (!empty($data['required']) ? ' required' : '');

		$result = '<select'.$attributes.$disabled.$required.'>'.$options_html.'</select>';

		return $result;
	}

	function render_textarea($data, $attributes, $field_type) {
		$textarea_value = $data['value'] ?? '';

		$disabled = isset($data[$field_type]['disabled']) ? '' : (!empty($data['disabled']) ? ' disabled' : '');
		$required = isset($data[$field_type]['required']) ? '' : (!empty($data['required']) ? ' required' : '');
		$result = '<textarea'.$attributes.$disabled.$required.'>'.$textarea_value.'</textarea>';

		return $result;
	}
}
