<?php

namespace core\taxonomy;
use core\{application\Application, database\Database, utility\Utility, theme\Theme, field\Field, route\Route, translation\Translation as T};

class Taxonomy {
	use Api, Form;

	function admin() {
		$taxonomy = Database::select('taxonomy_vocabulary')
			->fetch('assoc_all');
			
		foreach ($taxonomy as $key => $row) {
			$row['title'] = T::get($row['title']);
			$row['description'] = T::get($row['description']);
			
			$taxonomy[$key] = $row;
		}

		if ($taxonomy) {
			$data = [
				'taxonomy' => $taxonomy,
				'vocabulary_add' => T::get('Добавить словарь'),
				'name' => T::get('Название'),
				'title' => T::get('Заголовок'),
				'description' => T::get('Описание'),
				'number' => T::get('Номер'),
				'actions' => T::get('Действия'),
				'config' => T::get('настроить'),
				'see' => T::get('смотреть'),
				'delete' => T::get('удалить'),
				
				'add_route' => Route::render_route('admin/taxonomy/add'),
				'config_route' => Route::render_route('admin/field/list/taxonomy'),
				'terms_route' => Route::render_route('admin/taxonomy/terms'),
				'delete_route' => Route::render_route('admin/taxonomy/delete'),
			];

			$result = (new Theme)->get_template(__DIR__, 'admin', $data);
		} else {
			$result = '<a href="'.Route::render_route('admin/taxonomy/add').'">'.T::get('Добавить').'</a>
				<div class="margin-top">'.T::get('Словари отсутствуют').'</div>';
		}

		return $result;
	}

	function add_vocabulary() {
		Route::clear_post();

		$name = $_POST['name'] ?? '';
		$title = $_POST['title'] ?? '';
		$description = $_POST['description'] ?? '';

		$result = '';
		$save = false;
		if (isset($_POST['submit'])) {
			$reg_name = '/^([a-z_0-9]{3,50})$/';

			if (!preg_match($reg_name, $name)) {
				Utility::set_message('Проверьте поле "Название"', 'red');
			} else {
				$save = true;
			}
		}

		$data = compact('name', 'title', 'description');
		if (!$save) {
			$result = $this->get_form_vocabularies($data);
		} else {
			Database::insert('taxonomy_vocabulary')
				->data($data)
				->execute();

			Utility::set_message('Словарь успешно добавлен');
			Route::go_to('admin/taxonomy');
		}

		return $result;
	}

	function delete_vocabulary() {
		$id = Route::get_route_id();

		$result = '';
		if (isset($_POST['submit'])) {
			$ids = Database::select('field')
				->data(['id'])
				->where('entity', 'taxonomy')
				->fetch('assoc_map_all');

			$r_field_usage = Database::select('field_usage')
				->where('field_id', $ids)
				->fetch('assoc_all');

			foreach ($r_field_usage as $row) {
				$options = unserialize($row['options']);
				if ($options['vocabulary'] == $id) {
					Database::delete('field_usage')
						->where('id', $row['id'])
						->execute();
				}
			}

			$result_taxonomy_term = Database::select('taxonomy')
				->data(['id'])
				->where('vocabulary_id', $id)
				->fetch('assoc_all');

			if ($result_taxonomy_term) {
				foreach ($result_taxonomy_term as $row) {
					Database::delete('field_taxonomy')
						->where('value', $row['id'])
						->execute();

					Database::delete('route_aliases')
						->where('entity_id', $row['id'])
						->where('type', 'taxonomy')
						->execute();
				}

				Database::delete('taxonomy')
					->where('vocabulary_id', $id)
					->execute();
			}

			Database::delete('taxonomy_vocabulary')
				->where('id', $id)
				->execute();

			Utility::set_message('Словарь успешно удален', 'green');
			Route::go_to('admin/taxonomy');
		} else if (isset($_POST['cancel'])) {
			Route::go_to('admin/taxonomy');
		} else {
			$result_taxonomy = Database::select('taxonomy_vocabulary')
				->where('id', $id)
				->fetch('assoc');

			if ($result_taxonomy) {
				$result = Utility::get_form_delete('Также будет удалены все связанные термины и поля для материалов');
			}
		}

		return $result;
	}

	//

	function admin_terms() {
		$id = Route::get_route_id();

		$taxonomy = Database::select('taxonomy_vocabulary')
			->where('id', $id)
			->fetch('assoc');

		$taxonomy_term = Database::select('taxonomy')
			->where('vocabulary_id', $id)
			->fetch('assoc_all');

		foreach ($taxonomy_term as $key => $value) {
			$value['route'] = Route::render_route(Route::get_route_alias($value['id'], 'taxonomy'));
			$taxonomy_term[$key] = $value;
		}

		$result = '';
		if ($taxonomy) {
			if ($taxonomy_term) {
				$data = [
					'vocabulary_id' => $taxonomy['id'],
					'taxonomy_term' => $taxonomy_term,
					'vocabulary_name' => $taxonomy['name'],
					'term_add_in_vocabulary' => T::get('Добавить'),
					'number' => T::get('Номер'),
					'title' => T::get('Заголовок'),
					'actions' => T::get('Действия'),
					'see' => T::get('смотреть'),
					'edit' => T::get('изменить'),
					'delete' => T::get('удалить'),
					'vocabularies' => T::get('Словари'),
					
					'add_route' => Route::render_route('admin/taxonomy/terms/add'),
					'vocabularies_route' => Route::render_route('admin/taxonomy'),
					'see_route' => Route::render_route(''),
					'edit_route' => Route::render_route('admin/taxonomy/terms/edit'),
					'delete_route' => Route::render_route('admin/taxonomy/terms/delete'),
				];

				$result = (new Theme)->get_template(__DIR__, 'admin_terms', $data);
			} else {
				$result = '<a href="'.Route::render_route('admin/taxonomy/terms/add/'.$id).'">'.T::get('Добавить').'</a> |
					<a href="'.Route::render_route('admin/taxonomy').'">'.T::get('Словари').'</a>
					<div class="margin-top">'.T::get('Термины отсутствуют').'</div>';
			}
		}

		return $result;
	}

	//

	function add_term() {
		Route::clear_post();
		Utility::add_library(Route::get_path(__DIR__).'/js/taxonomy.js');

		$vocabulary_id = Route::get_route_id();

		$title = $_POST['title'] ?? '';
		$description = $_POST['description'] ?? '';
		$route_alias_auto = !isset($_POST['route_alias']) || isset($_POST['route_alias_auto']) ? 1 : 0;
		$route_alias = isset($_POST['route_alias_auto']) ?  T::get_translit($title) : ($_POST['route_alias'] ?? '');

		$result = '';
		if (isset($_POST['submit'])) {
			$reg_title = '/^([а-яёa-z0-9-., ]{3,100})$/iu';
			$reg_route = '/^([a-z0-9-\/]{3,100})$/';

			$result_route_alias = Database::select('route_aliases')
				->where('alias', $route_alias)
				->fetch('count');

			$message = '';
			if (preg_match($reg_route, $route_alias) && preg_match($reg_title, $title) && !$result_route_alias) {
				$term_id = Database::insert('taxonomy')
					->data(compact('title', 'description', 'vocabulary_id'))
					->fetch('field');

				Database::insert('route_aliases')
					->data([
						'auto' => $route_alias_auto,
						'alias' => $route_alias,
						'type' => 'taxonomy',
						'entity_id' => $term_id,
					])
					->execute();

				(new Field)->get_fields_usage('save', 'taxonomy', $vocabulary_id, $term_id);

				Utility::set_message('Термин успешно добавлен', 'green');
				Route::go_to('admin/taxonomy/terms/'.$vocabulary_id);
				return $result;
			} else {
				Utility::set_message('Проверьте поля', 'red');
			}
		}

		$result = $this->get_form_terms(compact('title', 'description', 'route_alias_auto', 'route_alias'), 'add', $vocabulary_id);

		return $result;
	}

	function edit_term() {
		$term_id = Route::get_route_id();

		$taxonomy_terms = Database::select('taxonomy')
			->where('id', $term_id)
			->fetch('object');

		$result = '';
		if ($taxonomy_terms) {
			$vocabulary_id = $taxonomy_terms->vocabulary_id;
			$route_alias_object = Route::get_route_alias($term_id, 'taxonomy', true);

			Route::clear_post();
			Utility::add_library(Route::get_path(__DIR__).'/js/taxonomy.js');

			$title = $_POST['title'] ?? $taxonomy_terms->title;
			$description = $_POST['description'] ?? $taxonomy_terms->description;
			$route_alias_auto = isset($_POST['route_alias_auto']) ? 1 : (isset($_POST['route_alias']) ? 0 : ($route_alias_object->auto ?? 1));
			$route_alias = isset($_POST['route_alias_auto']) ?  T::get_translit($title) : ($_POST['route_alias'] ?? ($route_alias_object->alias ?? ''));

			if (isset($_POST['submit'])) {
				$reg_title = '/^([а-яёa-z0-9-., ]{3,100})$/iu';
				$reg_route = '/^([a-z0-9-\/]{3,100})$/';

				$route_aliases = 0;
				if ($route_alias_object && $route_alias_object->alias != $route_alias) {
					$route_aliases = Database::select('route_aliases')
						->where('alias', $route_alias)
						->fetch('count');
				}

				$message = '';
				if (preg_match($reg_route, $route_alias) && preg_match($reg_title, $title) && !$route_aliases) {
					Database::update('taxonomy')
						->where('id', $term_id)
						->data(compact('title', 'description'))
						->execute();

					Database::update('route_aliases')
						->where('type', 'taxonomy')
						->where('entity_id', $term_id)
						->data([
							'auto' => $route_alias_auto,
							'alias' => $route_alias,
						])
						->execute();

					(new Field)->get_fields_usage('update', 'taxonomy', $vocabulary_id, $term_id);

					Utility::set_message('Термин успешно отредактирован');
					Route::go_to('admin/taxonomy/terms/'.$vocabulary_id);
					return $result;
				} else {
					Utility::set_message('Проверьте поля', 'red');
				}
			}

			$result = $this->get_form_terms(compact('title', 'description', 'route_alias_auto', 'route_alias'), 'edit', $vocabulary_id, $term_id);
		}

		return $result;
	}

	function delete_term() {
		$id = Route::get_route_id();

		$vocabulary_id = Database::select('taxonomy')
			->data(['vocabulary_id'])
			->where('id', $id)
			->fetch('field');

		$result = '';
		if (isset($_POST['submit'])) {
			(new Field)->get_fields_usage('delete', 'taxonomy', $vocabulary_id, $id);

			Database::delete('field_taxonomy')
				->where('value', $id)
				->execute();

			Database::delete('taxonomy')
				->where('id', $id)
				->execute();

			Database::delete('route_aliases')
				->where('entity_id', $id)
				->where('type', 'taxonomy')
				->execute();

			Utility::set_message('Термин успешно удален', 'green');
			Route::go_to('admin/taxonomy/terms/'.$vocabulary_id);
		} else if (isset($_POST['cancel'])) {
			Route::go_to('admin/taxonomy/terms/'.$vocabulary_id);
		} else {
			if ($vocabulary_id) {
				$result = Utility::get_form_delete();
			}
		}

		return $result;
	}

	function show_term() {
		$id = Route::get_route_id();
		
		$route_param = Route::get_route_language_param();
		$route_back = '/'.Route::get_route(true, 0, false).$route_param;
		
		$contents = [];

		$result = '';
		if ($id) {
			$term = Database::select('taxonomy')
				->data(['title', 'vocabulary_id',])
				->where('id', $id)
				->fetch('assoc');
			
			if ($term) {
				$vocabulary_name = Database::select('taxonomy_vocabulary')
					->data(['title',])
					->where('id', $term['vocabulary_id'])
					->fetch('field');
				
				$category_text = T::get($vocabulary_name).' / '.T::get($term['title']);
				$page_title = T::get('Список содержимого в категории').' "'.$category_text.'"';
				
				Application::set_title($page_title);

				$field_taxonomy = Database::select('field_taxonomy')
					->where('value', $id)
					->fetch('assoc_all');

				$field_taxonomy = array_reverse($field_taxonomy);
				if ($field_taxonomy) {
					$materials = [];

					foreach ($field_taxonomy as $key => $value) {
						$entity = Database::select('field_usage')
							->data(['type'])
							->where('id', $value['field_id'])
							->fetch('field');

						$entity_data = Database::select($entity)
							->where('id', $value['entity_id'])
							->fetch('assoc');

						$route_alias = $entity.'/'.$entity_data['id'];
						$title = $body = '';
						if ($entity == 'content') {
							$route_alias = Database::select('route_aliases')
								->data(['alias'])
								->where('entity_id', $entity_data['id'])
								->where('type', 'content')
								->fetch('field');

							$route_alias = Route::render_route(trim($route_alias, '/'));
							$title = T::get($entity_data['title']);
							$body = $entity_data['body'];
						} else if ($entity == 'user') {
							$title = $entity_data['login'];
							$body = $entity_data['name'];
						}						
						$title = T::get($title);
						$body_string = T::get($body);
						$route_alias .= $route_param;

						$title_short = mb_strimwidth($title, 0, 100, '...');
						$body_short = mb_strimwidth(strip_tags($body_string), 0, 150, '...');
						$date = date('d.m.y', $entity_data['changed'] ?: $entity_data['created']);

						preg_match('/src="([^"]+)"/i', $body, $matches);
						$image = '<img alt="'.T::get('Изображение отсутствует').'" src="'.Route::render_route('others/files/images/misc/no-image.jpg').'">';
						if (isset($matches[1])) {
							$image_src = $matches[1];
							$image_name = basename(mb_strcut($image_src, 0, -4));

							$image = '<img alt="'.$image_name.'" src="'.$image_src.'">';
						}
						
						$date_title = '';

						$contents[] = compact('route_alias', 'title', 'title_short', 'body_short', 'date_title', 'date', 'image');
					}

					$fields_data = (new Field)->get_fields_usage('show', 'taxonomy', $term['vocabulary_id'], $id);

					$fields = '';
					foreach ($fields_data as $key => $value) {
						if ($value) {
							$fields .= is_array($value) ? implode(', ', $value).', ' : $value.', ';
						}
					}
					$fields = rtrim($fields, ', ');

					$result = $fields ? '<div class="field">'.$fields.'</div>' : '';
				}
			}
		}
		
		$strings = [
			'back' => T::get('Назад'),
			'empty' => T::get('Содержимое, помеченное данным тегом, отсутствует'),
		];
		
		$result = $result.(new Theme)->get_template(__DIR__, 'show_term', compact('contents', 'strings', 'route_back'));
		
		return $result;
	}
}
